import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';

import 'package:scrobbles/cubit/data_counts_by_day_cubit.dart';
import 'package:scrobbles/cubit/data_counts_by_hour_cubit.dart';
import 'package:scrobbles/cubit/data_discoveries_cubit.dart';
import 'package:scrobbles/cubit/data_statistics_global_cubit.dart';
import 'package:scrobbles/cubit/data_statistics_recent_cubit.dart';
import 'package:scrobbles/cubit/data_timeline_cubit.dart';
import 'package:scrobbles/cubit/data_top_artists_cubit.dart';
import 'package:scrobbles/models/counts_by_day.dart';
import 'package:scrobbles/models/counts_by_hour.dart';
import 'package:scrobbles/models/discoveries.dart';
import 'package:scrobbles/models/statistics_global.dart';
import 'package:scrobbles/models/statistics_recent.dart';
import 'package:scrobbles/models/timeline.dart';
import 'package:scrobbles/models/topartists.dart';
import 'package:scrobbles/network/scrobbles.dart';
import 'package:scrobbles/ui/widgets/error.dart';

class UpdateData extends StatelessWidget {
  const UpdateData({super.key});

  final int daysCount = 21;
  final Widget loading = const Text('⏳');
  final Widget done = const Text('');

  @override
  Widget build(BuildContext context) {
    return Padding(
      padding: EdgeInsets.all(4),
      child: Stack(
        children: [
          updateCountsByDay(daysCount),
          updateCountsByHour(daysCount),
          updateDiscoveries(daysCount),
          updateStatisticsGlobal(),
          updateStatisticsRecent(daysCount),
          updateTimeline(daysCount),
          updateTopArtists(daysCount),
        ],
      ),
    );
  }

  Widget updateCountsByDay(int daysCount) {
    late Future<CountsByDayData> futureCountsByDay = ScrobblesApi.fetchCountsByDay(daysCount);

    return BlocProvider<DataCountsByDayCubit>(
      create: (BuildContext context) => DataCountsByDayCubit(),
      child: BlocBuilder<DataCountsByDayCubit, DataCountsByDayState>(
        builder: (BuildContext context, DataCountsByDayState state) {
          return FutureBuilder<CountsByDayData>(
            future: futureCountsByDay,
            builder: (context, snapshot) {
              if (snapshot.hasError) {
                return ShowErrorWidget(message: '${snapshot.error}');
              }

              BlocProvider.of<DataCountsByDayCubit>(context).setValue(snapshot.data);
              return !snapshot.hasData ? loading : done;
            },
          );
        },
      ),
    );
  }

  Widget updateCountsByHour(int daysCount) {
    late Future<CountsByHourData> futureCountsByHour =
        ScrobblesApi.fetchCountsByHour(daysCount);

    return BlocProvider<DataCountsByHourCubit>(
      create: (BuildContext context) => DataCountsByHourCubit(),
      child: BlocBuilder<DataCountsByHourCubit, DataCountsByHourState>(
        builder: (BuildContext context, DataCountsByHourState state) {
          return FutureBuilder<CountsByHourData>(
            future: futureCountsByHour,
            builder: (context, snapshot) {
              if (snapshot.hasError) {
                return ShowErrorWidget(message: '${snapshot.error}');
              }

              BlocProvider.of<DataCountsByHourCubit>(context).setValue(snapshot.data);
              return !snapshot.hasData ? loading : done;
            },
          );
        },
      ),
    );
  }

  Widget updateDiscoveries(int daysCount) {
    late Future<DiscoveriesData> futureDiscoveries = ScrobblesApi.fetchDiscoveries(daysCount);

    return BlocProvider<DataDiscoveriesCubit>(
      create: (BuildContext context) => DataDiscoveriesCubit(),
      child: BlocBuilder<DataDiscoveriesCubit, DataDiscoveriesState>(
        builder: (BuildContext context, DataDiscoveriesState state) {
          return FutureBuilder<DiscoveriesData>(
            future: futureDiscoveries,
            builder: (context, snapshot) {
              if (snapshot.hasError) {
                return ShowErrorWidget(message: '${snapshot.error}');
              }

              BlocProvider.of<DataDiscoveriesCubit>(context).setValue(snapshot.data);
              return !snapshot.hasData ? loading : done;
            },
          );
        },
      ),
    );
  }

  Widget updateStatisticsGlobal() {
    late Future<StatisticsGlobalData> futureStatisticsGlobal =
        ScrobblesApi.fetchGlobalStatistics();

    return BlocProvider<DataStatisticsGlobalCubit>(
      create: (BuildContext context) => DataStatisticsGlobalCubit(),
      child: BlocBuilder<DataStatisticsGlobalCubit, DataStatisticsGlobalState>(
        builder: (BuildContext context, DataStatisticsGlobalState dataState) {
          return FutureBuilder<StatisticsGlobalData>(
            future: futureStatisticsGlobal,
            builder: (context, snapshot) {
              if (snapshot.hasError) {
                return ShowErrorWidget(message: '${snapshot.error}');
              }

              BlocProvider.of<DataStatisticsGlobalCubit>(context).setValue(snapshot.data);
              return !snapshot.hasData ? loading : done;
            },
          );
        },
      ),
    );
  }

  Widget updateStatisticsRecent(int daysCount) {
    late Future<StatisticsRecentData> futureStatisticsRecent =
        ScrobblesApi.fetchRecentStatistics(daysCount);

    return BlocProvider<DataStatisticsRecentCubit>(
      create: (BuildContext context) => DataStatisticsRecentCubit(),
      child: BlocBuilder<DataStatisticsRecentCubit, DataStatisticsRecentState>(
        builder: (BuildContext context, DataStatisticsRecentState state) {
          return FutureBuilder<StatisticsRecentData>(
            future: futureStatisticsRecent,
            builder: (context, snapshot) {
              if (snapshot.hasError) {
                return ShowErrorWidget(message: '${snapshot.error}');
              }

              BlocProvider.of<DataStatisticsRecentCubit>(context).setValue(snapshot.data);
              return !snapshot.hasData ? loading : done;
            },
          );
        },
      ),
    );
  }

  Widget updateTimeline(int daysCount) {
    late Future<TimelineData> futureTimeline = ScrobblesApi.fetchTimeline(daysCount);

    return BlocProvider<DataTimelineCubit>(
      create: (BuildContext context) => DataTimelineCubit(),
      child: BlocBuilder<DataTimelineCubit, DataTimelineState>(
        builder: (BuildContext context, DataTimelineState state) {
          return FutureBuilder<TimelineData>(
            future: futureTimeline,
            builder: (context, snapshot) {
              if (snapshot.hasError) {
                return ShowErrorWidget(message: '${snapshot.error}');
              }

              BlocProvider.of<DataTimelineCubit>(context).setValue(snapshot.data);
              return !snapshot.hasData ? loading : done;
            },
          );
        },
      ),
    );
  }

  Widget updateTopArtists(int daysCount) {
    late Future<TopArtistsData> futureTopArtists = ScrobblesApi.fetchTopArtists(daysCount);

    return BlocProvider<DataTopArtistsCubit>(
      create: (BuildContext context) => DataTopArtistsCubit(),
      child: BlocBuilder<DataTopArtistsCubit, DataTopArtistsState>(
        builder: (BuildContext context, DataTopArtistsState state) {
          return FutureBuilder<TopArtistsData>(
            future: futureTopArtists,
            builder: (context, snapshot) {
              if (snapshot.hasError) {
                return ShowErrorWidget(message: '${snapshot.error}');
              }

              BlocProvider.of<DataTopArtistsCubit>(context).setValue(snapshot.data);
              return !snapshot.hasData ? loading : done;
            },
          );
        },
      ),
    );
  }
}
